package com.mortals.xhx.module.one.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.xhx.busiz.oneProblemlist.OneProblemInfo;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.one.model.*;
import com.mortals.xhx.module.one.service.OneClassifyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.one.dao.OneClassifyTopicalDao;
import com.mortals.xhx.module.one.service.OneClassifyTopicalService;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>Title: 分类引导主题中间表</p>
 * <p>Description: OneClassifyTopicalService service接口 </p>
 * <p>Copyright: Copyright &reg;  </p>
 *
 * @version 1.0.0
 */
@Service("oneClassifyTopicalService")
public class OneClassifyTopicalServiceImpl extends AbstractCRUDServiceImpl<OneClassifyTopicalDao, OneClassifyTopicalEntity, Long> implements OneClassifyTopicalService {

    @Autowired
    private OneClassifyService oneClassifyService;


    @Override
    protected void saveBefore(OneClassifyTopicalEntity entity, Context context) throws AppException {
        int max = this.find(new OneClassifyTopicalQuery()).stream().mapToInt(OneClassifyTopicalEntity::getClassifyRank).max().orElseGet(() -> 0);
        entity.setClassifyRank(max + 1);
        super.saveBefore(entity, context);
    }

    @Override
    protected void logicDeleteAfter(OneClassifyTopicalEntity entity, Context context) throws AppException {
        //逻辑级联删除
        List<OneClassifyEntity> oneClassifyEntities = oneClassifyService.find(new OneClassifyQuery().deleted(YesNoEnum.NO.getValue()).parentId(entity.getClassifyId()));
        if (!ObjectUtils.isEmpty(oneClassifyEntities)) {
            oneClassifyEntities.stream().peek(item -> oneClassifyService.logicDelete(item, context)).count();
        }
        super.logicDeleteAfter(entity, context);
    }

    @Override
    public void upOrDownTopicList(Long id, Integer type) {

        OneClassifyTopicalQuery query = new OneClassifyTopicalQuery().deleted(YesNoEnum.NO.getValue());
        query.setOrderColList(Arrays.asList(new OrderCol("classify_rank")));
        query.setOrderKind(OrderCol.ASCENDING);
        List<OneClassifyTopicalEntity> list = this.find(query);
        OneClassifyTopicalEntity preEntity = null;
        OneClassifyTopicalEntity curEntity = null;
        if (type == YesNoEnum.NO.getValue()) {
            //up
            for (int i = 0; i < list.size(); i++) {
                if (list.get(i).getId() == id && i > 0) {
                    curEntity = list.get(i);
                    preEntity = list.get(i - 1);
                    break;
                }
            }

        } else {
            //down
            for (int i = 0; i < list.size(); i++) {
                if (list.get(i).getId() == id && i + 1 < list.size()) {
                    curEntity = list.get(i);
                    preEntity = list.get(i + 1);
                    break;
                }
            }
        }

        if (preEntity != null && curEntity != null) {
            //交换
            int temp = preEntity.getClassifyRank();
            preEntity.setClassifyRank(curEntity.getClassifyRank());
            curEntity.setClassifyRank(temp);

            this.update(preEntity);
            this.update(curEntity);
        }


    }

    @Override
    public OneProblemInfo findAllQuestion(Long id) {
        //获取根节点，递归获取所有树节点

        OneProblemInfo info = new OneProblemInfo();

        OneClassifyEntity rootClassify = oneClassifyService.get(id);
        if (!ObjectUtils.isEmpty(rootClassify)) {

            info.setId(rootClassify.getId());
            info.setIsLeaf(rootClassify.getIsLeaf());
            info.setLevel(rootClassify.getClassifyLevel());
            info.setMatterIds(rootClassify.getMatterIds());
            info.setName(rootClassify.getClassifyName());
            info.setParentId(rootClassify.getParentId());
            info.setRemark(rootClassify.getSortRemarks());
            info.setIsAns(rootClassify.getIsAns());


            List<OneProblemInfo> collect = getOneClassifyEntities(rootClassify);

            if(!ObjectUtils.isEmpty(collect)){
                info.setChildren(collect);
            }

        }

        return info;

    }

    private List<OneProblemInfo> getOneClassifyEntities(OneClassifyEntity rootClassify) {
        List<OneProblemInfo> collect = oneClassifyService.find(new OneClassifyQuery().parentId(rootClassify.getId()).deleted(YesNoEnum.NO.getValue()))
                .stream().map(item -> {
                    OneProblemInfo info = new OneProblemInfo();
                    info.setId(item.getId());
                    info.setIsLeaf(item.getIsLeaf());
                    info.setLevel(item.getClassifyLevel());
                    info.setMatterIds(item.getMatterIds());
                    info.setName(item.getClassifyName());
                    info.setParentId(item.getParentId());
                    info.setRemark(item.getSortRemarks());
                    info.setIsAns(item.getIsAns());

                    List<OneProblemInfo> oneClassifyEntities = this.getOneClassifyEntities(item);

                    if(!ObjectUtils.isEmpty(oneClassifyEntities)){
                        info.setChildren(oneClassifyEntities);
                    }

                    return info;
                }).collect(Collectors.toList());
        return collect;
    }
}